@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "CommandLine.hxx"
@@ -36,12 +20,12 @@
 #include "fs/NarrowPath.hxx"
 #include "fs/Traits.hxx"
 #include "fs/FileSystem.hxx"
-#include "fs/StandardDirectory.hxx"
+#include "fs/glue/StandardDirectory.hxx"
 #include "event/Features.h"
 #include "io/uring/Features.h"
+#include "cmdline/OptionDef.hxx"
+#include "cmdline/OptionParser.hxx"
 #include "util/Domain.hxx"
-#include "util/OptionDef.hxx"
-#include "util/OptionParser.hxx"
 #include "Version.h"
 
 #ifdef _WIN32
@@ -71,6 +55,8 @@
 #include "archive/ArchivePlugin.hxx"
 #endif
 
+#include <fmt/core.h>
+
 namespace {
 #ifdef _WIN32
 constexpr auto CONFIG_FILE_LOCATION = Path::FromFS(PATH_LITERAL("mpd\\mpd.conf"));
@@ -86,6 +72,9 @@ enum Option {
 	OPTION_KILL,
 	OPTION_NO_CONFIG,
 	OPTION_NO_DAEMON,
+#ifdef __linux__
+	OPTION_SYSTEMD,
+#endif
 	OPTION_STDOUT,
 	OPTION_STDERR,
 	OPTION_VERBOSE,
@@ -98,6 +87,9 @@ static constexpr OptionDef option_defs[] = {
 	{"kill", "kill the currently running mpd session"},
 	{"no-config", "don't read from config"},
 	{"no-daemon", "don't detach from console"},
+#ifdef __linux__
+	{"systemd", "systemd service mode"},
+#endif
 	{"stdout", nullptr}, // hidden, compatibility with old versions
 	{"stderr", "print messages to stderr"},
 	{"verbose", 'v', "verbose logging"},
@@ -111,159 +103,166 @@ static constexpr Domain cmdline_domain("cmdline");
 [[noreturn]]
 static void version()
 {
-	printf("Music Player Daemon " VERSION " (%s)"
-	       "\n"
-	       "Copyright 2003-2007 Warren Dukes <warren.dukes@gmail.com>\n"
-	       "Copyright 2008-2021 Max Kellermann <max.kellermann@gmail.com>\n"
-	       "This is free software; see the source for copying conditions.  There is NO\n"
-	       "warranty; not even MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
-	       GIT_VERSION);
+	fmt::print("Music Player Daemon " VERSION " ({})"
+		   "\n"
+		   "Copyright 2003-2007 Warren Dukes <warren.dukes@gmail.com>\n"
+		   "Copyright 2008-2021 Max Kellermann <max.kellermann@gmail.com>\n"
+		   "This is free software; see the source for copying conditions.  There is NO\n"
+		   "warranty; not even MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
+		   GIT_VERSION);
 
 #ifdef ENABLE_DATABASE
-	printf("\n"
-	       "Database plugins:\n");
+	fmt::print("\n"
+		   "Database plugins:\n");
 
 	for (auto i = database_plugins; *i != nullptr; ++i)
-		printf(" %s", (*i)->name);
+		fmt::print(" {}", (*i)->name);
 
-	printf("\n\n"
-	       "Storage plugins:\n");
+	fmt::print("\n\n"
+		   "Storage plugins:\n");
 
 	for (auto i = storage_plugins; *i != nullptr; ++i)
-		printf(" %s", (*i)->name);
+		fmt::print(" {}", (*i)->name);
 
-	printf("\n");
+	fmt::print("\n");
 #endif
 
 #ifdef ENABLE_NEIGHBOR_PLUGINS
-	printf("\n"
-	       "Neighbor plugins:\n");
+	fmt::print("\n"
+		   "Neighbor plugins:\n");
 	for (auto i = neighbor_plugins; *i != nullptr; ++i)
-		printf(" %s", (*i)->name);
+		fmt::print(" {}", (*i)->name);
 
 #endif
 
-	printf("\n"
-	       "\n"
-	       "Decoders plugins:\n");
+	fmt::print("\n"
+		   "\n"
+		   "Decoder plugins:\n");
 
-	decoder_plugins_for_each([](const DecoderPlugin &plugin){
-		printf(" [%s]", plugin.name);
+	for (const DecoderPlugin &plugin : GetAllDecoderPlugins()) {
+		fmt::print(" [{}]", plugin.name);
 
 		const char *const*suffixes = plugin.suffixes;
 		if (suffixes != nullptr)
 			for (; *suffixes != nullptr; ++suffixes)
-				printf(" %s", *suffixes);
+				fmt::print(" {}", *suffixes);
+
+		if (plugin.suffixes_function != nullptr)
+			for (const auto &i : plugin.suffixes_function())
+				printf(" %s", i.c_str());
 
 		if (plugin.protocols != nullptr)
 			for (const auto &i : plugin.protocols())
-				printf(" %s", i.c_str());
+				fmt::print(" {}", i);
 
-		printf("\n");
-	});
-
-	printf("\n"
-	       "Filters:\n"
-#ifdef ENABLE_LIBSAMPLERATE
-	       " libsamplerate"
-#endif
-#ifdef ENABLE_SOXR
-	       " soxr"
-#endif
-	       "\n\n"
-	       "Tag plugins:\n"
-#ifdef ENABLE_ID3TAG
-	       " id3tag"
-#endif
-	       "\n\n"
-	       "Output plugins:\n");
-	audio_output_plugins_for_each(plugin)
-		printf(" %s", plugin->name);
-	printf("\n"
-
-#ifdef ENABLE_ENCODER
-	       "\n"
-	       "Encoder plugins:\n");
-	encoder_plugins_for_each(plugin)
-		printf(" %s", plugin->name);
-	printf("\n"
-#endif
-
-#ifdef ENABLE_ARCHIVE
-	       "\n"
-	       "Archive plugins:\n");
-	archive_plugins_for_each(plugin) {
-		printf(" [%s]", plugin->name);
-
-		const char *const*suffixes = plugin->suffixes;
-		if (suffixes != nullptr)
-			for (; *suffixes != nullptr; ++suffixes)
-				printf(" %s", *suffixes);
-
-		printf("\n");
+		fmt::print("\n");
 	}
 
-	printf(""
+	fmt::print("\n"
+		   "Filters:\n"
+#ifdef ENABLE_LIBSAMPLERATE
+		   " libsamplerate"
+#endif
+#ifdef ENABLE_SOXR
+		   " soxr"
+#endif
+		   "\n\n"
+		   "Tag plugins:\n"
+#ifdef ENABLE_ID3TAG
+		   " id3tag"
+#endif
+		   "\n\n"
+		   "Output plugins:\n");
+	for (const auto &plugin : GetAllAudioOutputPlugins()) {
+		fmt::print(" {}", plugin.name);
+	}
+	fmt::print("\n"
+
+#ifdef ENABLE_ENCODER
+		   "\n"
+		   "Encoder plugins:\n");
+	for (const auto &plugin : GetAllEncoderPlugins()) {
+		fmt::print(" {}", plugin.name);
+	}
+	fmt::print("\n"
 #endif
 
-	       "\n"
-	       "Input plugins:\n"
-	       " file"
+#ifdef ENABLE_ARCHIVE
+		   "\n"
+		   "Archive plugins:\n");
+	for (const auto &plugin : GetAllArchivePlugins()) {
+		fmt::print(" [{}]", plugin.name);
+
+		const char *const*suffixes = plugin.suffixes;
+		if (suffixes != nullptr)
+			for (; *suffixes != nullptr; ++suffixes)
+				fmt::print(" {}", *suffixes);
+
+		fmt::print("\n");
+	}
+
+	fmt::print(""
+#endif
+
+		   "\n"
+		   "Input plugins:\n"
+		   " file"
 #ifdef HAVE_URING
-	       " io_uring"
+		   " io_uring"
 #endif
 #ifdef ENABLE_ARCHIVE
-	       " archive"
+		   " archive"
 #endif
-	       );
-	input_plugins_for_each(plugin)
-		printf(" %s", plugin->name);
+		);
+	for (const InputPlugin &plugin : GetAllInputPlugins())
+		fmt::print(" {}", plugin.name);
 
-	printf("\n\n"
-	       "Playlist plugins:\n");
-	playlist_plugins_for_each(plugin)
-		printf(" %s", plugin->name);
+	fmt::print("\n\n"
+		   "Playlist plugins:\n");
+	for (const auto &plugin : GetAllPlaylistPlugins()) {
+		fmt::print(" {}", plugin.name);
+	}
 
-	printf("\n\n"
-	       "Protocols:\n");
+	fmt::print("\n\n"
+		   "Protocols:\n");
 	print_supported_uri_schemes_to_fp(stdout);
 
-	printf("\n"
-	       "Other features:\n"
+	fmt::print("\n"
+		   "Other features:\n"
 #ifdef HAVE_AVAHI
-	       " avahi"
+		   " avahi"
 #endif
 #ifdef ENABLE_DBUS
-	       " dbus"
+		   " dbus"
 #endif
 #ifdef ENABLE_UDISKS
-	       " udisks"
+		   " udisks"
 #endif
 #ifdef USE_EPOLL
-	       " epoll"
+		   " epoll"
 #endif
 #ifdef HAVE_ICONV
-	       " iconv"
+		   " iconv"
 #endif
 #ifdef HAVE_ICU
-	       " icu"
+		   " icu"
 #endif
 #ifdef ENABLE_INOTIFY
-	       " inotify"
+		   " inotify"
 #endif
 #ifdef HAVE_IPV6
-	       " ipv6"
+		   " ipv6"
 #endif
 #ifdef ENABLE_SYSTEMD_DAEMON
-	       " systemd"
+		   " systemd"
 #endif
 #ifdef HAVE_TCP
-	       " tcp"
+		   " tcp"
 #endif
 #ifdef HAVE_UN
-	       " un"
+		   " un"
 #endif
-	       "\n");
+		   "\n");
 
 	std::exit(EXIT_SUCCESS);
 }
@@ -284,12 +283,12 @@ static void PrintOption(const OptionDef &opt)
 [[noreturn]]
 static void help()
 {
-	printf("Usage:\n"
-	       "  mpd [OPTION...] [path/to/mpd.conf]\n"
-	       "\n"
-	       "Music Player Daemon - a daemon for playing music.\n"
-	       "\n"
-	       "Options:\n");
+	fmt::print("Usage:\n"
+		   "  mpd [OPTION...] [path/to/mpd.conf]\n"
+		   "\n"
+		   "Music Player Daemon - a daemon for playing music.\n"
+		   "\n"
+		   "Options:\n");
 
 	for (const auto &i : option_defs)
 		if(i.HasDescription()) // hide hidden options from help print
@@ -328,7 +327,7 @@ bool ConfigLoader::TryFile(const AllocatedPath &base_path, Path path)
 }
 
 void
-ParseCommandLine(int argc, char **argv, struct options &options,
+ParseCommandLine(int argc, char **argv, CommandLineOptions &options,
 		 ConfigData &config)
 {
 	bool use_config_file = true;
@@ -346,9 +345,20 @@ ParseCommandLine(int argc, char **argv, struct options &options,
 			break;
 
 		case OPTION_NO_DAEMON:
+#ifdef ENABLE_DAEMON
 			options.daemon = false;
+#endif
 			break;
 
+#ifdef __linux__
+		case OPTION_SYSTEMD:
+#ifdef ENABLE_DAEMON
+			options.daemon = false;
+#endif
+			options.systemd = true;
+			break;
+#endif
+
 		case OPTION_STDOUT:
 		case OPTION_STDERR:
 			options.log_stderr = true;
