@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "LogBackend.hxx"
 #include "Log.hxx"
@@ -92,13 +76,15 @@ EnableLogTimestamp() noexcept
 static const char *
 log_date() noexcept
 {
-	static constexpr size_t LOG_DATE_BUF_SIZE = 16;
+	static constexpr size_t LOG_DATE_BUF_SIZE = std::char_traits<char>::length("Jan 22 15:43:14 : ") + 1;
 	static char buf[LOG_DATE_BUF_SIZE];
 	time_t t = time(nullptr);
-	strftime(buf, LOG_DATE_BUF_SIZE, "%b %d %H:%M : ", localtime(&t));
+	strftime(buf, LOG_DATE_BUF_SIZE, "%b %d %H:%M:%S : ", localtime(&t));
 	return buf;
 }
 
+#ifdef HAVE_SYSLOG
+
 /**
  * Determines the length of the string excluding trailing whitespace
  * characters.
@@ -109,8 +95,6 @@ chomp_length(std::string_view p) noexcept
 	return StripRight(p.data(), p.size());
 }
 
-#ifdef HAVE_SYSLOG
-
 [[gnu::const]]
 static int
 ToSysLogLevel(LogLevel log_level) noexcept
@@ -163,10 +147,10 @@ LogFinishSysLog() noexcept
 static void
 FileLog(const Domain &domain, std::string_view message) noexcept
 {
-	fprintf(stderr, "%s%s: %.*s\n",
-		enable_timestamp ? log_date() : "",
-		domain.GetName(),
-		chomp_length(message), message.data());
+	fmt::print(stderr, "{}{}: {}\n",
+		   enable_timestamp ? log_date() : "",
+		   domain.GetName(),
+		   StripRight(message));
 
 #ifdef _WIN32
 	/* force-flush the log file, because setvbuf() does not seem
