@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "FlacDecoderPlugin.h"
 #include "FlacStreamDecoder.hxx"
@@ -24,15 +8,13 @@
 #include "lib/xiph/FlacMetadataChain.hxx"
 #include "OggCodec.hxx"
 #include "input/InputStream.hxx"
+#include "input/LocalOpen.hxx"
 #include "fs/Path.hxx"
 #include "fs/NarrowPath.hxx"
 #include "Log.hxx"
 
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-#error libFLAC is too old
-#endif
-
-static void flacPrintErroredState(FLAC__StreamDecoderState state)
+static void
+flacPrintErroredState(FLAC__StreamDecoderState state) noexcept
 {
 	switch (state) {
 	case FLAC__STREAM_DECODER_SEARCH_FOR_METADATA:
@@ -40,6 +22,9 @@ static void flacPrintErroredState(FLAC__StreamDecoderState state)
 	case FLAC__STREAM_DECODER_SEARCH_FOR_FRAME_SYNC:
 	case FLAC__STREAM_DECODER_READ_FRAME:
 	case FLAC__STREAM_DECODER_END_OF_STREAM:
+#if FLAC_API_VERSION_CURRENT >= 14
+	case FLAC__STREAM_DECODER_END_OF_LINK:
+#endif
 		return;
 
 	case FLAC__STREAM_DECODER_OGG_ERROR:
@@ -53,8 +38,9 @@ static void flacPrintErroredState(FLAC__StreamDecoderState state)
 	LogError(flac_domain, FLAC__StreamDecoderStateString[state]);
 }
 
-static void flacMetadata([[maybe_unused]] const FLAC__StreamDecoder * dec,
-			 const FLAC__StreamMetadata * block, void *vdata)
+static void
+flacMetadata([[maybe_unused]] const FLAC__StreamDecoder * dec,
+	     const FLAC__StreamMetadata * block, void *vdata) noexcept
 {
 	auto &fd = *(FlacDecoder *)vdata;
 	fd.OnMetadata(*block);
@@ -62,29 +48,46 @@ static void flacMetadata([[maybe_unused]] const FLAC__StreamDecoder * dec,
 
 static FLAC__StreamDecoderWriteStatus
 flac_write_cb(const FLAC__StreamDecoder *dec, const FLAC__Frame *frame,
-	      const FLAC__int32 *const buf[], void *vdata)
+	      const FLAC__int32 *const buf[], void *vdata) noexcept
 {
 	auto &fd = *(FlacDecoder *)vdata;
 	return fd.OnWrite(*frame, buf, fd.GetDeltaPosition(*dec));
 }
 
 static bool
-flac_scan_file(Path path_fs, TagHandler &handler)
+flac_scan_file(Path path_fs, TagHandler &handler) noexcept
 {
 	FlacMetadataChain chain;
-	if (!chain.Read(NarrowPath(path_fs))) {
+	const bool succeed = [&chain, &path_fs]() noexcept {
+		// read by NarrowPath
+		if (chain.Read(NarrowPath(path_fs))) {
+			return true;
+		}
+		if (std::is_same_v<Path::value_type, char> ||
+		    chain.GetStatus() != FLAC__METADATA_CHAIN_STATUS_ERROR_OPENING_FILE) {
+			return false;
+		}
+		// read by InputStream
+		Mutex mutex;
+		auto is = OpenLocalInputStream(path_fs, mutex);
+		if (is && chain.Read(*is)) {
+			return true;
+		}
+		return false;
+	}();
+
+	if (!succeed) {
 		FmtDebug(flac_domain,
 			 "Failed to read FLAC tags: {}",
 			 chain.GetStatusString());
 		return false;
 	}
-
 	chain.Scan(handler);
 	return true;
 }
 
 static bool
-flac_scan_stream(InputStream &is, TagHandler &handler)
+flac_scan_stream(InputStream &is, TagHandler &handler) noexcept
 {
 	FlacMetadataChain chain;
 	if (!chain.Read(is)) {
@@ -102,7 +105,7 @@ flac_scan_stream(InputStream &is, TagHandler &handler)
  * Some glue code around FLAC__stream_decoder_new().
  */
 static FlacStreamDecoder
-flac_decoder_new()
+flac_decoder_new() noexcept
 {
 	FlacStreamDecoder sd;
 	if(!FLAC__stream_decoder_set_metadata_respond(sd.get(), FLAC__METADATA_TYPE_VORBIS_COMMENT))
@@ -113,7 +116,7 @@ flac_decoder_new()
 }
 
 static bool
-flac_decoder_initialize(FlacDecoder *data, FLAC__StreamDecoder *sd)
+flac_decoder_initialize(FlacDecoder *data, FLAC__StreamDecoder *sd) noexcept
 {
 	if (!FLAC__stream_decoder_process_until_end_of_metadata(sd)) {
 		if (FLAC__stream_decoder_get_state(sd) != FLAC__STREAM_DECODER_END_OF_STREAM)
@@ -152,11 +155,9 @@ FlacSubmitToClient(DecoderClient &client, FlacDecoder &d) noexcept
 	}
 
 	if (!d.chunk.empty()) {
-		auto cmd = client.SubmitData(d.GetInputStream(),
-					     d.chunk.data,
-					     d.chunk.size,
-					     d.kbit_rate);
-		d.chunk = nullptr;
+		auto cmd = client.SubmitAudio(d.GetInputStream(), d.chunk,
+					      d.kbit_rate);
+		d.chunk = {};
 		if (cmd != DecoderCommand::NONE)
 			return cmd;
 	}
@@ -218,6 +219,12 @@ flac_decoder_loop(FlacDecoder *data, FLAC__StreamDecoder *flac_dec)
 		case FLAC__STREAM_DECODER_UNINITIALIZED:
 			/* we shouldn't see this, ever - bail out */
 			return;
+
+#if FLAC_API_VERSION_CURRENT >= 14
+		case FLAC__STREAM_DECODER_END_OF_LINK:
+			/* TODO support FLAC 1.5 chained streams */
+			return;
+#endif
 		}
 
 		if (!FLAC__stream_decoder_process_single(flac_dec) &&
@@ -231,7 +238,7 @@ flac_decoder_loop(FlacDecoder *data, FLAC__StreamDecoder *flac_dec)
 }
 
 static FLAC__StreamDecoderInitStatus
-stream_init_oggflac(FLAC__StreamDecoder *flac_dec, FlacDecoder *data)
+stream_init_oggflac(FLAC__StreamDecoder *flac_dec, FlacDecoder *data) noexcept
 {
 	return FLAC__stream_decoder_init_ogg_stream(flac_dec,
 						    FlacInput::Read,
@@ -246,7 +253,7 @@ stream_init_oggflac(FLAC__StreamDecoder *flac_dec, FlacDecoder *data)
 }
 
 static FLAC__StreamDecoderInitStatus
-stream_init_flac(FLAC__StreamDecoder *flac_dec, FlacDecoder *data)
+stream_init_flac(FLAC__StreamDecoder *flac_dec, FlacDecoder *data) noexcept
 {
 	return FLAC__stream_decoder_init_stream(flac_dec,
 						FlacInput::Read,
@@ -261,7 +268,8 @@ stream_init_flac(FLAC__StreamDecoder *flac_dec, FlacDecoder *data)
 }
 
 static FLAC__StreamDecoderInitStatus
-stream_init(FLAC__StreamDecoder *flac_dec, FlacDecoder *data, bool is_ogg)
+stream_init(FLAC__StreamDecoder *flac_dec, FlacDecoder *data,
+	    bool is_ogg) noexcept
 {
 	return is_ogg
 		? stream_init_oggflac(flac_dec, data)
@@ -307,7 +315,7 @@ flac_decode(DecoderClient &client, InputStream &input_stream)
 }
 
 static bool
-oggflac_init([[maybe_unused]] const ConfigBlock &block)
+oggflac_init([[maybe_unused]] const ConfigBlock &block) noexcept
 {
 	return !!FLAC_API_SUPPORTS_OGG_FLAC;
 }
