@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "AsyncInputStream.hxx"
 #include "tag/Tag.hxx"
@@ -26,7 +10,7 @@
 
 #include <string.h>
 
-AsyncInputStream::AsyncInputStream(EventLoop &event_loop, const char *_url,
+AsyncInputStream::AsyncInputStream(EventLoop &event_loop, std::string_view _url,
 				   Mutex &_mutex,
 				   size_t _buffer_size,
 				   size_t _resume_at) noexcept
@@ -34,9 +18,9 @@ AsyncInputStream::AsyncInputStream(EventLoop &event_loop, const char *_url,
 	 deferred_resume(event_loop, BIND_THIS_METHOD(DeferredResume)),
 	 deferred_seek(event_loop, BIND_THIS_METHOD(DeferredSeek)),
 	 allocation(_buffer_size),
-	 buffer(&allocation.front(), allocation.size()),
 	 resume_at(_resume_at)
 {
+	allocation.SetName("InputStream");
 	allocation.ForkCow(false);
 }
 
@@ -122,9 +106,9 @@ AsyncInputStream::Seek(std::unique_lock<Mutex> &lock,
 			break;
 
 		const size_t nbytes =
-			new_offset - offset < (offset_type)r.size
-					       ? new_offset - offset
-					       : r.size;
+			std::cmp_less(new_offset - offset, r.size())
+			? new_offset - offset
+			: r.size();
 
 		buffer.Consume(nbytes);
 		offset += nbytes;
@@ -175,30 +159,12 @@ AsyncInputStream::IsAvailable() const noexcept
 		!buffer.empty();
 }
 
-size_t
-AsyncInputStream::Read(std::unique_lock<Mutex> &lock,
-		       void *ptr, size_t read_size)
+inline std::size_t
+AsyncInputStream::ReadFromBuffer(std::span<std::byte> dest) noexcept
 {
-	assert(!GetEventLoop().IsInside());
-
-	/* wait for data */
-	CircularBuffer<uint8_t>::Range r;
-	while (true) {
-		Check();
-
-		r = buffer.Read();
-		if (!r.empty())
-			break;
-
-		if (IsEOF())
-			return 0;
-
-		caller_cond.wait(lock);
-	}
-
-	const size_t nbytes = std::min(read_size, r.size);
-	memcpy(ptr, r.data, nbytes);
-	buffer.Consume(nbytes);
+	const size_t nbytes = buffer.MoveTo(dest);
+	if (nbytes == 0)
+		return 0;
 
 	if (buffer.empty())
 		/* when the buffer becomes empty, reset its head and
@@ -207,13 +173,33 @@ AsyncInputStream::Read(std::unique_lock<Mutex> &lock,
 		buffer.Clear();
 
 	offset += (offset_type)nbytes;
-
-	if (paused && buffer.GetSize() < resume_at)
-		deferred_resume.Schedule();
-
 	return nbytes;
 }
 
+size_t
+AsyncInputStream::Read(std::unique_lock<Mutex> &lock,
+		       std::span<std::byte> dest)
+{
+	assert(!GetEventLoop().IsInside());
+
+	/* wait for data */
+	while (true) {
+		Check();
+
+		if (std::size_t nbytes = ReadFromBuffer(dest); nbytes > 0) {
+			if (paused && buffer.GetSize() < resume_at)
+				deferred_resume.Schedule();
+
+			return nbytes;
+		}
+
+		if (IsEOF())
+			return 0;
+
+		caller_cond.wait(lock);
+	}
+}
+
 void
 AsyncInputStream::CommitWriteBuffer(size_t nbytes) noexcept
 {
@@ -228,23 +214,28 @@ AsyncInputStream::CommitWriteBuffer(size_t nbytes) noexcept
 }
 
 void
-AsyncInputStream::AppendToBuffer(const void *data, size_t append_size) noexcept
+AsyncInputStream::AppendToBuffer(std::span<const std::byte> src) noexcept
 {
 	auto w = buffer.Write();
 	assert(!w.empty());
 
-	size_t nbytes = std::min(w.size, append_size);
-	memcpy(w.data, data, nbytes);
-	buffer.Append(nbytes);
+	std::span<const std::byte> second{};
 
-	const size_t remaining = append_size - nbytes;
-	if (remaining > 0) {
+	if (w.size() < src.size()) {
+		second = src.subspan(w.size());
+		src = src.first(w.size());
+	}
+
+	std::copy(src.begin(), src.end(), w.begin());
+	buffer.Append(src.size());
+
+	if (!second.empty()) {
 		w = buffer.Write();
 		assert(!w.empty());
-		assert(w.size >= remaining);
+		assert(w.size() >= second.size());
 
-		memcpy(w.data, (const uint8_t *)data + nbytes, remaining);
-		buffer.Append(remaining);
+		std::copy(second.begin(), second.end(), w.begin());
+		buffer.Append(second.size());
 	}
 
 	if (!IsReady())
@@ -258,7 +249,15 @@ AsyncInputStream::AppendToBuffer(const void *data, size_t append_size) noexcept
 void
 AsyncInputStream::DeferredResume() noexcept
 {
-	const std::scoped_lock<Mutex> protect(mutex);
+	const std::scoped_lock protect{mutex};
+
+	if (postponed_exception) {
+		/* do not proceed, first the caller must handle the
+                   pending error */
+		caller_cond.notify_one();
+		InvokeOnAvailable();
+		return;
+	}
 
 	try {
 		Resume();
@@ -272,10 +271,18 @@ AsyncInputStream::DeferredResume() noexcept
 void
 AsyncInputStream::DeferredSeek() noexcept
 {
-	const std::scoped_lock<Mutex> protect(mutex);
+	const std::scoped_lock protect{mutex};
 	if (seek_state != SeekState::SCHEDULED)
 		return;
 
+	if (postponed_exception) {
+		/* do not proceed, first the caller must handle the
+                   pending error */
+		caller_cond.notify_one();
+		InvokeOnAvailable();
+		return;
+	}
+
 	try {
 		Resume();
 
