@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Source.hxx"
 #include "MusicChunk.hxx"
@@ -23,16 +7,13 @@
 #include "filter/Prepared.hxx"
 #include "filter/plugins/ReplayGainFilterPlugin.hxx"
 #include "pcm/Mix.hxx"
+#include "lib/fmt/AudioFormatFormatter.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "thread/Mutex.hxx"
-#include "util/ConstBuffer.hxx"
-#include "util/RuntimeError.hxx"
 
 #include <string.h>
 
-AudioOutputSource::AudioOutputSource() noexcept
-{
-}
-
+AudioOutputSource::AudioOutputSource() noexcept = default;
 AudioOutputSource::~AudioOutputSource() noexcept = default;
 
 AudioFormat
@@ -72,9 +53,9 @@ AudioOutputSource::Close() noexcept
 	assert(in_audio_format.IsValid());
 	in_audio_format.Clear();
 
-	Cancel();
-
 	CloseFilter();
+
+	Cancel();
 }
 
 void
@@ -93,7 +74,7 @@ AudioOutputSource::Cancel() noexcept
 		filter->Reset();
 }
 
-void
+inline void
 AudioOutputSource::OpenFilter(AudioFormat audio_format,
 			      PreparedFilter *prepared_replay_gain_filter,
 			      PreparedFilter *prepared_other_replay_gain_filter,
@@ -133,7 +114,7 @@ AudioOutputSource::CloseFilter() noexcept
 	filter.reset();
 }
 
-ConstBuffer<void>
+std::span<const std::byte>
 AudioOutputSource::GetChunkData(const MusicChunk &chunk,
 				Filter *current_replay_gain_filter,
 				unsigned *replay_gain_serial_p)
@@ -141,9 +122,9 @@ AudioOutputSource::GetChunkData(const MusicChunk &chunk,
 	assert(!chunk.IsEmpty());
 	assert(chunk.CheckFormat(in_audio_format));
 
-	ConstBuffer<void> data(chunk.data, chunk.length);
+	auto data = chunk.ReadData();
 
-	assert(data.size % in_audio_format.GetFrameSize() == 0);
+	assert(data.size() % in_audio_format.GetFrameSize() == 0);
 
 	if (!data.empty() && current_replay_gain_filter != nullptr) {
 		replay_gain_filter_set_mode(*current_replay_gain_filter,
@@ -157,15 +138,19 @@ AudioOutputSource::GetChunkData(const MusicChunk &chunk,
 			*replay_gain_serial_p = chunk.replay_gain_serial;
 		}
 
+		/* note: the ReplayGainFilter doesn't have a
+		   ReadMore() method */
 		data = current_replay_gain_filter->FilterPCM(data);
 	}
 
 	return data;
 }
 
-ConstBuffer<void>
+inline std::span<const std::byte>
 AudioOutputSource::FilterChunk(const MusicChunk &chunk)
 {
+	assert(filter);
+
 	auto data = GetChunkData(chunk, replay_gain_filter.get(),
 				 &replay_gain_serial);
 	if (data.empty())
@@ -185,8 +170,8 @@ AudioOutputSource::FilterChunk(const MusicChunk &chunk)
 		   "next" song being faded in, and if there's a rest,
 		   it means cross-fading ends here */
 
-		if (data.size > other_data.size)
-			data.size = other_data.size;
+		if (data.size() > other_data.size())
+			data = data.first(other_data.size());
 
 		float mix_ratio = chunk.mix_ratio;
 		if (mix_ratio >= 0)
@@ -197,16 +182,15 @@ AudioOutputSource::FilterChunk(const MusicChunk &chunk)
 			   case */
 			mix_ratio = 1.0f - mix_ratio;
 
-		void *dest = cross_fade_buffer.Get(other_data.size);
-		memcpy(dest, other_data.data, other_data.size);
-		if (!pcm_mix(cross_fade_dither, dest, data.data, data.size,
+		void *dest = cross_fade_buffer.Get(other_data.size());
+		memcpy(dest, other_data.data(), other_data.size());
+		if (!pcm_mix(cross_fade_dither, dest, data.data(), data.size(),
 			     in_audio_format.format,
 			     mix_ratio))
-			throw FormatRuntimeError("Cannot cross-fade format %s",
-						 sample_format_to_string(in_audio_format.format));
+			throw FmtRuntimeError("Cannot cross-fade format {}",
+					      in_audio_format.format);
 
-		data.data = dest;
-		data.size = other_data.size;
+		data = {(const std::byte *)dest, other_data.size()};
 	}
 
 	/* apply filter chain */
@@ -217,6 +201,8 @@ AudioOutputSource::FilterChunk(const MusicChunk &chunk)
 bool
 AudioOutputSource::Fill(Mutex &mutex)
 {
+	assert(filter);
+
 	if (current_chunk != nullptr && pending_tag == nullptr &&
 	    pending_data.empty())
 		DropCurrentChunk();
@@ -235,7 +221,7 @@ AudioOutputSource::Fill(Mutex &mutex)
 		   that may take a while */
 		const ScopeUnlock unlock(mutex);
 
-		pending_data = pending_data.FromVoid(FilterChunk(*current_chunk));
+		pending_data = FilterChunk(*current_chunk);
 	} catch (...) {
 		current_chunk = nullptr;
 		throw;
@@ -247,16 +233,24 @@ AudioOutputSource::Fill(Mutex &mutex)
 void
 AudioOutputSource::ConsumeData(size_t nbytes) noexcept
 {
-	pending_data.skip_front(nbytes);
+	assert(filter);
 
-	if (pending_data.empty())
-		DropCurrentChunk();
+	pending_data = pending_data.subspan(nbytes);
+
+	if (pending_data.empty()) {
+		/* give the filter a chance to return more data in
+		   another buffer */
+		pending_data = filter->ReadMore();
+
+		if (pending_data.empty())
+			DropCurrentChunk();
+	}
 }
 
-ConstBuffer<void>
+std::span<const std::byte>
 AudioOutputSource::Flush()
 {
-	return filter
-		? filter->Flush()
-		: nullptr;
+	assert(filter);
+
+	return filter->Flush();
 }
