@@ -1,27 +1,14 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "PlaylistMapper.hxx"
 #include "PlaylistFile.hxx"
+#include "PlaylistRegistry.hxx"
 #include "PlaylistStream.hxx"
 #include "SongEnumerator.hxx"
 #include "Mapper.hxx"
+#include "input/InputStream.hxx"
+#include "input/WaitReady.hxx"
 #include "fs/AllocatedPath.hxx"
 #include "storage/StorageInterface.hxx"
 #include "util/UriUtil.hxx"
@@ -49,21 +36,19 @@ playlist_open_in_playlist_dir(const char *uri, Mutex &mutex)
  * Load a playlist from the configured music directory.
  */
 static std::unique_ptr<SongEnumerator>
-playlist_open_in_storage(const char *uri, const Storage *storage, Mutex &mutex)
+playlist_open_in_storage(const char *uri, Storage *storage, Mutex &mutex)
 {
 	assert(uri_safe_local(uri));
 
 	if (storage == nullptr)
 		return nullptr;
 
-	{
-		const auto path = storage->MapFS(uri);
-		if (!path.IsNull())
-			return playlist_open_path(path, mutex);
-	}
+	if (const auto path = storage->MapFS(uri); !path.IsNull())
+		return playlist_open_path(path, mutex);
 
-	const auto uri2 = storage->MapUTF8(uri);
-	return playlist_open_remote(uri2.c_str(), mutex);
+	auto is = storage->OpenFile(uri, mutex);
+	LockWaitReady(*is);
+	return playlist_list_open_stream(std::move(is), uri);
 }
 
 #endif
@@ -71,14 +56,22 @@ playlist_open_in_storage(const char *uri, const Storage *storage, Mutex &mutex)
 std::unique_ptr<SongEnumerator>
 playlist_mapper_open(const char *uri,
 #ifdef ENABLE_DATABASE
-		     const Storage *storage,
+		     Storage *storage,
 #endif
 		     Mutex &mutex)
 {
+	std::exception_ptr spl_error;
+
 	if (spl_valid_name(uri)) {
-		auto playlist = playlist_open_in_playlist_dir(uri, mutex);
-		if (playlist != nullptr)
-			return playlist;
+		try {
+			auto playlist = playlist_open_in_playlist_dir(uri, mutex);
+			if (playlist != nullptr)
+				return playlist;
+		} catch (...) {
+			/* postpone this exception, try playlist in
+			   music_directory first */
+			spl_error = std::current_exception();
+		}
 	}
 
 #ifdef ENABLE_DATABASE
@@ -89,5 +82,8 @@ playlist_mapper_open(const char *uri,
 	}
 #endif
 
+	if (spl_error)
+		std::rethrow_exception(spl_error);
+
 	return nullptr;
 }
